from gemini_framework.abstract.unit_module_abstract import UnitModuleAbstract
from gemini_model.pump.esp import ESP
import numpy as np
import traceback


class CalculateTheoreticalPumpPower(UnitModuleAbstract):
    model = None

    def __init__(self, unit):
        super().__init__(unit)

        self.model = ESP()

    def link(self):
        self.link_input(self.unit, 'measured', 'esp_flow')
        self.link_input(self.unit, 'measured', 'esp_frequency')
        self.link_output(self.unit, 'calculated', 'esp_power')

    def step(self, loop):
        try:
            self.loop = loop
            self.loop.start_time = self.get_output_last_data_time('esp_power')
            self.loop.compute_n_simulation()

            time, esp_flow = self.get_input_data('esp_flow')
            time, esp_frequency = self.get_input_data('esp_frequency')

            u = dict()
            esp_pump_power = []
            time_calc = []
            for ii in range(1, self.loop.n_step + 1):
                self.update_model_parameter(time[ii])

                if (esp_flow[ii] is None) or (esp_frequency[ii] is None):
                    esp_pump_power.append(None)
                    continue

                u['pump_flow'] = esp_flow[ii] / 3600  # convert to seconds
                u['pump_freq'] = esp_frequency[ii]

                x = []
                self.model.calculate_output(u, x)

                y = self.model.get_output()

                esp_pump_power.append(y['pump_power'])
                time_calc.append(time[ii])

            if time_calc:
                self.write_output_data('esp_power', time_calc, esp_pump_power)

        except Exception:
            self.logger.warn(
                "ERROR in module " + self.__class__.__name__ + " : " + traceback.format_exc())

    def update_model_parameter(self, timestamp):

        index = self.get_parameter_index(self.unit, timestamp)

        esp_param = dict()
        esp_param['no_stages'] = self.unit.parameters['property']['esp_no_stage'][index]
        esp_param['pump_name'] = self.unit.parameters['property']['esp_type'][index]
        esp_param['head_coeff'] = np.asarray(
            self.unit.parameters['property']['esp_head_coeff'][index].split(';'), dtype=np.float32)
        esp_param['power_coeff'] = np.asarray(
            self.unit.parameters['property']['esp_power_coeff'][index].split(';'), dtype=np.float32)

        self.model.update_parameters(esp_param)
